@@ -1,6 +1,5 @@
 import json
 import os
-from packaging.version import parse as parse_version
 import tempfile
 
 import numpy as np
@@ -10,14 +9,19 @@ import strax
 import straxen
 from straxen.common import pax_file, get_resource, first_sr1_run
 export, __all__ = strax.exporter()
-
+from .pulse_processing import  HE_PREAMBLE
 
 @export
 @strax.takes_config(
-    strax.Option('n_top_pmts', default=127,
+    strax.Option('n_top_pmts', default=straxen.n_top_pmts,
                  help="Number of top PMTs"))
 class PeakBasics(strax.Plugin):
-    __version__ = "0.0.5"
+    """
+    Compute the basic peak-properties, thereby dropping structured
+    arrays.
+    NB: This plugin can therefore be loaded as a pandas DataFrame.
+    """
+    __version__ = "0.0.7"
     parallel = True
     depends_on = ('peaks',)
     provides = 'peak_basics'
@@ -35,12 +39,13 @@ class PeakBasics(strax.Plugin):
         (('PMT number which contributes the most PE',
             'max_pmt'), np.int16),
         (('Area of signal in the largest-contributing PMT (PE)',
-            'max_pmt_area'), np.int32),
+            'max_pmt_area'), np.float32),
         (('Width (in ns) of the central 50% area of the peak',
             'range_50p_area'), np.float32),
         (('Width (in ns) of the central 90% area of the peak',
             'range_90p_area'), np.float32),
-        (('Fraction of area seen by the top array',
+        (('Fraction of area seen by the top array '
+          '(NaN for peaks with non-positive area)',
             'area_fraction_top'), np.float32),
         (('Length of the peak waveform in samples',
           'length'), np.int32),
@@ -71,10 +76,11 @@ class PeakBasics(strax.Plugin):
 
         n_top = self.config['n_top_pmts']
         area_top = p['area_per_channel'][:, :n_top].sum(axis=1)
-        # Negative-area peaks get 0 AFT - TODO why not NaN?
+        # Negative-area peaks get NaN AFT
         m = p['area'] > 0
         r['area_fraction_top'][m] = area_top[m]/p['area'][m]
-        r['rise_time'] = -p['area_decile_from_midpoint'][:,1]
+        r['area_fraction_top'][~m] = float('nan')
+        r['rise_time'] = -p['area_decile_from_midpoint'][:, 1]
         return r
 
     @staticmethod
@@ -89,6 +95,18 @@ class PeakBasics(strax.Plugin):
         return result
 
 
+@export
+class PeakBasicsHighEnergy(PeakBasics):
+    __doc__ = HE_PREAMBLE + PeakBasics.__doc__
+    __version__ = '0.0.1'
+    depends_on = 'peaks_he'
+    provides = 'peak_basics_he'
+    child_ends_with = '_he'
+
+    def compute(self, peaks_he):
+        return super().compute(peaks_he)
+
+
 @export
 @strax.takes_config(
     strax.Option(
@@ -106,10 +124,11 @@ class PeakBasics(strax.Plugin):
     strax.Option('min_reconstruction_area',
                  help='Skip reconstruction if area (PE) is less than this',
                  default=10),
-    strax.Option('n_top_pmts', default=127,
+    strax.Option('n_top_pmts', default=straxen.n_top_pmts,
                  help="Number of top PMTs")
 )
 class PeakPositions(strax.Plugin):
+    """Compute the S2 (x,y)-position based on a neural net."""
     dtype = [('x', np.float32,
               'Reconstructed S2 X position (cm), uncorrected'),
              ('y', np.float32,
@@ -196,6 +215,10 @@ class PeakPositions(strax.Plugin):
                  help='Maximum value for proximity values such as '
                       't_to_next_peak [ns]'))
 class PeakProximity(strax.OverlapWindowPlugin):
+    """
+    Look for peaks around a peak to determine how many peaks are in
+    proximity (in time) of a peak.
+    """
     depends_on = ('peak_basics',)
     dtype = [
         ('n_competing', np.int32,
