@@ -1,3 +1,4 @@
+from immutabledict import immutabledict
 import numba
 import numpy as np
 
@@ -5,15 +6,27 @@ import strax
 import straxen
 
 export, __all__ = strax.exporter()
-
+__all__ += ['NO_PULSE_COUNTS']
 
 # These are also needed in peaklets, since hitfinding is repeated
 HITFINDER_OPTIONS = tuple([
     strax.Option(
         'hit_min_amplitude',
-        default=15,
+        default='pmt_commissioning_initial',
         help='Minimum hit amplitude in ADC counts above baseline. '
-             'Specify as a tuple of length n_tpc_pmts, or a number.')])
+             'See straxen.hit_min_amplitude for options.'
+    )])
+
+HITFINDER_OPTIONS_he = tuple([
+    strax.Option(
+        'hit_min_amplitude_he', track=True,
+        default="pmt_commissioning_initial_he",
+        child_option=True,
+        help='Minimum hit amplitude in ADC counts above baseline for the high energy channels. '
+             'See straxen.hit_min_amplitude for options.'
+    )])
+
+HE_PREAMBLE = """High energy channels: attenuated signals of the top PMT-array\n"""
 
 
 @export
@@ -74,24 +87,33 @@ HITFINDER_OPTIONS = tuple([
         default=False, track=False,
         help=('Use a default baseline for incorrectly chunked fragments. '
               'This is a kludge for improperly converted XENON1T data.')),
-    
-    *HITFINDER_OPTIONS)
 
+    *HITFINDER_OPTIONS)
 class PulseProcessing(strax.Plugin):
     """
     1. Split raw_records into:
-     - tpc_records
+     - (tpc) records
      - aqmon_records
+     - pulse_counts
 
     For TPC records, apply basic processing:
-    1. Flip, baseline, and integrate the waveform
-    2. Apply software HE veto after high-energy peaks.
-    3. Find hits, apply linear filter, and zero outside hits.
+        1. Flip, baseline, and integrate the waveform
+        2. Apply software HE veto after high-energy peaks.
+        3. Find hits, apply linear filter, and zero outside hits.
+    
+    pulse_counts holds some average information for the individual PMT
+    channels for each chunk of raw_records. This includes e.g.
+    number of recorded pulses, lone_pulses (pulses which do not
+    overlap with any other pulse), or mean values of baseline and
+    baseline rms channel.
     """
-    __version__ = '0.2.0'
+    __version__ = '0.2.2'
 
     parallel = 'process'
-    rechunk_on_save = False
+    rechunk_on_save = immutabledict(
+        records=False,
+        veto_regions=True,
+        pulse_counts=True)
     compressor = 'lz4'
 
     depends_on = 'raw_records'
@@ -115,15 +137,15 @@ class PulseProcessing(strax.Plugin):
 
     def setup(self):
         self.hev_enabled = (
-            (self.config['hev_gain_model'][0] != 'disabled')
-            and self.config['tail_veto_threshold'])
+                (self.config['hev_gain_model'][0] != 'disabled')
+                and self.config['tail_veto_threshold'])
         if self.hev_enabled:
             self.to_pe = straxen.get_to_pe(
                 self.run_id,
                 self.config['hev_gain_model'],
                 n_tpc_pmts=self.config['n_tpc_pmts'])
 
-    def compute(self, raw_records):
+    def compute(self, raw_records, start, end):
         if self.config['check_raw_record_overlaps']:
             check_overlaps(raw_records, n_channels=3000)
 
@@ -149,6 +171,8 @@ class PulseProcessing(strax.Plugin):
         strax.integrate(r)
 
         pulse_counts = count_pulses(r, self.config['n_tpc_pmts'])
+        pulse_counts['time'] = start
+        pulse_counts['endtime'] = end
 
         if len(r) and self.hev_enabled:
             r, r_vetoed, veto_regions = software_he_veto(
@@ -171,7 +195,8 @@ class PulseProcessing(strax.Plugin):
             # -- before filtering,since this messes with the with the S/N
             hits = strax.find_hits(
                 r,
-                min_amplitude=self.config['hit_min_amplitude'])
+                min_amplitude=straxen.hit_min_amplitude(
+                    self.config['hit_min_amplitude']))
 
             if self.config['pmt_pulse_filter']:
                 # Filter to concentrate the PMT pulses
@@ -190,11 +215,47 @@ class PulseProcessing(strax.Plugin):
                     pulse_counts=pulse_counts,
                     veto_regions=veto_regions)
 
+    
+@export
+@strax.takes_config(
+    strax.Option('n_he_pmts', track=False, default=752,
+                 help="Maximum channel of the he channels"),
+    strax.Option('record_length', default=110, track=False, type=int,
+                 help="Number of samples per raw_record"),
+    *HITFINDER_OPTIONS_he)
+class PulseProcessingHighEnergy(PulseProcessing):
+    __doc__ = HE_PREAMBLE + PulseProcessing.__doc__
+    __version__ = '0.0.1'
+    provides = ('records_he', 'pulse_counts_he')
+    data_kind = {k: k for k in provides}
+    rechunk_on_save = immutabledict(
+        records_he=False,
+        pulse_counts_he=True)
+    depends_on = 'raw_records_he'
+    compressor = 'lz4'
+    child_ends_with = '_he'
+
+    def infer_dtype(self):
+        dtype = dict()
+        dtype['records_he'] = strax.record_dtype(self.config["record_length"])
+        dtype['pulse_counts_he'] = pulse_count_dtype(self.config['n_he_pmts'])
+        return dtype
+
+    def setup(self):
+        self.hev_enabled = False
+        self.config['n_tpc_pmts'] = self.config['n_he_pmts']
+        self.config['hit_min_amplitude'] = self.config['hit_min_amplitude_he']
+
+    def compute(self, raw_records_he, start, end):
+        result = super().compute(raw_records_he, start, end)
+        return dict(records_he=result['records'],
+                    pulse_counts_he=result['pulse_counts'])
 
 ##
 # Software HE Veto
 ##
 
+
 @export
 def software_he_veto(records, to_pe,
                      area_threshold=int(1e5),
@@ -297,7 +358,7 @@ def software_he_veto(records, to_pe,
 
     # 5. Apply the veto and return results
     veto_mask = strax.fully_contained_in(records, veto) == -1
-    return tuple(list(_mask_and_not(records, veto_mask)) + [veto])
+    return tuple(list(mask_and_not(records, veto_mask)) + [veto])
 
 
 @numba.njit(cache=True, nogil=True)
@@ -339,14 +400,13 @@ def rough_sum(regions, records, to_pe, n, dt):
 ##
 # Pulse counting
 ##
-
 @export
 def pulse_count_dtype(n_channels):
     # NB: don't use the dt/length interval dtype, integer types are too small
     # to contain these huge chunk-wide intervals
     return [
-        (('Lowest start time observed in the chunk', 'time'), np.int64),
-        (('Highest endt ime observed in the chunk', 'endtime'), np.int64),
+        (('Start time of the chunk', 'time'), np.int64),
+        (('End time of the chunk', 'endtime'), np.int64),
         (('Number of pulses', 'pulse_count'),
          (np.int64, n_channels)),
         (('Number of lone pulses', 'lone_pulse_count'),
@@ -355,6 +415,10 @@ def pulse_count_dtype(n_channels):
          (np.int64, n_channels)),
         (('Integral of lone pulses in ADC_count x samples', 'lone_pulse_area'),
          (np.int64, n_channels)),
+        (('Average baseline', 'baseline_mean'),
+         (np.int16, n_channels)),
+        (('Average baseline rms', 'baseline_rms_mean'),
+         (np.float32, n_channels)),
     ]
 
 
@@ -367,6 +431,7 @@ def count_pulses(records, n_channels):
     return np.zeros(0, dtype=pulse_count_dtype(n_channels))
 
 
+NO_PULSE_COUNTS = -9999  # Special value required by average_baseline in case counts = 0
 @numba.njit(cache=True, nogil=True)
 def _count_pulses(records, n_channels, result):
     count = np.zeros(n_channels, dtype=np.int64)
@@ -380,20 +445,23 @@ def _count_pulses(records, n_channels, result):
     # Array of booleans to track whether we are currently in a lone pulse
     # in each channel
     in_lone_pulse = np.zeros(n_channels, dtype=np.bool_)
-
+    baseline_buffer = np.zeros(n_channels, dtype=np.float64)
+    baseline_rms_buffer = np.zeros(n_channels, dtype=np.float64)
     for r_i, r in enumerate(records):
         if r_i != len(records) - 1:
             next_start = records[r_i + 1]['time']
 
         ch = r['channel']
         if ch >= n_channels:
-            print(ch)
+            print('Channel:', ch)
             raise RuntimeError("Out of bounds channel in get_counts!")
 
         area[ch] += r['area']  # <-- Summing total area in channel
 
         if r['record_i'] == 0:
             count[ch] += 1
+            baseline_buffer[ch] += r['baseline']
+            baseline_rms_buffer[ch] += r['baseline_rms'] 
 
             if (r['time'] > last_end_seen
                     and r['time'] + r['pulse_length'] * r['dt'] < next_start):
@@ -416,16 +484,18 @@ def _count_pulses(records, n_channels, result):
     res['lone_pulse_count'][:] = lone_count[:]
     res['pulse_area'][:] = area[:]
     res['lone_pulse_area'][:] = lone_area[:]
-    res['time'] = records[0]['time']
-    res['endtime'] = last_end_seen
+    means = (baseline_buffer/count)
+    means[np.isnan(means)] = NO_PULSE_COUNTS
+    res['baseline_mean'][:] = means[:]
+    res['baseline_rms_mean'][:] = (baseline_rms_buffer/count)[:]
 
 
 ##
 # Misc
 ##
-
+@export
 @numba.njit(cache=True, nogil=True)
-def _mask_and_not(x, mask):
+def mask_and_not(x, mask):
     return x[mask], x[~mask]
 
 
@@ -433,7 +503,7 @@ def _mask_and_not(x, mask):
 @numba.njit(cache=True, nogil=True)
 def channel_split(rr, first_other_ch):
     """Return """
-    return _mask_and_not(rr, rr['channel'] < first_other_ch)
+    return mask_and_not(rr, rr['channel'] < first_other_ch)
 
 
 @export
